#include <ESP8266WiFi.h>
#include <UrsNTPTimeSource.h>


bool UrsNTPTimeSource::addNTPServer(String ntpServername) {
  if (ntpServername.length()) {
    if (servers < maxServers) {
      serverNames[servers] = ntpServername;
      //    serverReaminingLoops[servers] = 0;
      servers++;
      return true;
    }
  }
  return false;
}

// ToDo: Reihenfolge ndern, wenn ein Server nicht geantwortet hat
// besser: Immer nur einen Server pro Anfrage und dies steuerbar
/* Zeitregelung erfolgt auerhalb
UrsTimeSourceState UrsNTPTimeSource::loadTime() {
  for (uint8_t i = 0; i < servers; i++) {
    if (serverReaminingLoops[i] == 0) { // Server ist nicht gesperrt, ...
      if (getNtpTime(serverNames[i])) { // ... dann Server abfragen, ...
        return UrsTimeSourceState::success;
      }
      else { // Server bei nicht erfolgreicher Abfrage sperren
        serverReaminingLoops[i] = retryAfter;
        return UrsTimeSourceState::timeout;
      }
    }
    else
      serverReaminingLoops[i]--;        // ... ansonsten Sperrzeit verringern
  }
  return UrsTimeSourceState::timeout;
}
*/

UrsTimeSourceState UrsNTPTimeSource::loadTime() {
  for (uint8_t i = 0; i < servers; i++) {
    if (getNtpTime(serverNames[i])) { // Server abfragen
      return UrsTimeSourceState::success;
    }else{
    }
  } // for
  return UrsTimeSourceState::timeout;
}

bool UrsNTPTimeSource::getNtpTime(String ntpServerName) {
  IPAddress ntpServerIP; // IP-Adresse des NTP Servers
  WiFi.hostByName(ntpServerName.c_str(), ntpServerIP); // IP-Adresse des Servers ermitteln

  Udp.begin(NTPPort);

  while (Udp.parsePacket() > 0); // evtl. bereits empfangene Daten lschen

  //NTP-Anfrage-Packet versenden
  DEBUGT("Transmit NTP Request: %s, %s\n", ntpServerName.c_str(), ntpServerIP.toString().c_str());
  memset(packetBuffer, 0, NTP_PACKET_SIZE); // Puffer lschen
                                            // NTP-Anfrage-Paket aufbauen
  packetBuffer[0] = 0b11100011;   // LI, Version, Mode
  packetBuffer[1] = 0;     // Stratum, or type of clock
  packetBuffer[2] = 6;     // Polling Interval
  packetBuffer[3] = 0xEC;  // Peer Clock Precision
  packetBuffer[12] = 49;
  packetBuffer[13] = 0x4E;
  packetBuffer[14] = 49;
  packetBuffer[15] = 52;
  // Paket versenden
  Udp.beginPacket(ntpServerIP, NTPPort);
  Udp.write(packetBuffer, NTP_PACKET_SIZE);
#if DEBUG_URS_TIME
  int rtc = 
#endif
     Udp.endPacket();
  DEBUGT("Udp.endpackt: %i\n", rtc);

  uint32_t beginWait = millis();
  while (millis() - beginWait < timeOut) {
    int size = Udp.parsePacket();
    if (size >= NTP_PACKET_SIZE) {
      DEBUGT("Receive NTP Response\n");
      Udp.read(packetBuffer, NTP_PACKET_SIZE);  // read packet into the buffer
      unsigned long secsSince1900;
      // convert four bytes starting at location 40 to a long integer
      secsSince1900 = (unsigned long)packetBuffer[40] << 24;
      secsSince1900 |= (unsigned long)packetBuffer[41] << 16;
      secsSince1900 |= (unsigned long)packetBuffer[42] << 8;
      secsSince1900 |= (unsigned long)packetBuffer[43];
      current = secsSince1900 - 2208988800UL;
      Udp.stop();
      return true;
    }
  }
  DEBUGT("Error: NTP Timeout\n");
  Udp.stop();
  return false; // return 0 if unable to get the time
}